@@ -137,6 +137,7 @@ class ZbxCommon(object):
             self.modules['kafka'] = 'kafka-python==2.0.2'
             self.modules['zmq'] = 'pyzmq==23.2.0'
             self.modules['flask'] = 'Flask==2.0.1'
+            self.modules['pymysql'] = 'PyMySQL==1.0.2'
             self.modules['elasticsearch'] = 'elasticsearch==8.3.1'
         if len(sys.argv) == 2 and len(sys.argv[1]) == 36:
             self.get_param('')
@@ -149,7 +150,12 @@ class ZbxCommon(object):
             with open(path + '/requirement.txt', 'w+') as fp:
                 fp.write("\n".join(mods))
         else:
-            print("\n".join(mods))
+            if PY3:
+                for m in mods:
+                    print('pip3 install %s' % (m))
+            else:
+                for m in mods:
+                    print('pip install %s' % (m))
 
     def init_log(self):
         pass
@@ -972,17 +978,70 @@ class ZbxCommon(object):
             else:
                 return cursor.execute(sql, args)
 
-    def dict2sql(self, table_name, table_data):
+    def _safe_dict(self, data):
+        data = dict(data)
+        for k, v in data.items():
+            data[k] = self._safe_value(v)
+        return data
+
+    def _safe_list(self, data):
+        if not isinstance(data, list):
+            return data
+        data = list(data)
+        for i, v in enumerate(data):
+            if type(v) == dict:
+                data[i] = self.mysql_safe_dict(v)
+            else:
+                data[i] = self._safe_value(v)
+
+    def mysql_safe_value(self, v):
+        if isinstance(v, list):
+            return self._safe_list(v)
+        if isinstance(v, dict):
+            return self._safe_dict(v)
+        return self._safe_value(v)
+
+    def _safe_value(self, v):
+        if type(v) == int or type(v) == float:
+            return str(v)
+        if v is None:
+            return 'NULL'
+        v = str(v).replace('\\', '\\\\').replace("'", "\\'")
+        v = v.replace("\n", "\\n").replace('"', '\\"')
+        return v
+
+    def gen_update_sql(self, table_name, dict_data, where_dict=None):
+        vs = []
+        dict_data = dict(dict_data)
+        where = ''
+        if where_dict is None and 'id' not in dict_data.keys():
+            return 'not support'
+        if where_dict is None and 'id' in dict_data.keys():
+            where = ' WHERE id={id}'.format(id=self._safe_value(dict_data['id']))
+            del dict_data['id']
+        if where_dict is not None:
+            ws = []
+            where_dict = dict(where_dict)
+            for k, v in where_dict.items():
+                ws.append("`{k}`='{v}'".format(k=k, v=self._safe_value(v)))
+                where = ' WHERE {where}'.format(where=' AND '.join(ws))
+        for k, v in dict_data.items():
+            vs.append("`{k}`='{v}'".format(k=k, v=v))
+        sql = '''UPDATE `{table_name}` SET {fields} {where}'''.format(table_name=table_name,
+                                                                      fields=','.join(vs), where=where)
+        return sql
+
+    def gen_insert_sql(self, table_name, list_data):
         row = {}
-        if type(table_data) != list:
+        if type(list_data) != list:
             self.log.error("table_data is not list,name: {table_name}".format(table_name=table_name))
             return ''
-        if len(table_data) > 0:
-            row = table_data[0]
+        if len(list_data) > 0:
+            row = list_data[0]
         fields = list(row.keys())
         # map(lambda x:'`%s`'%(x),fields)
         rows = []
-        for row in table_data:
+        for row in list_data:
             data = []
             for fn in fields:
                 if row[fn] is None:
@@ -1023,7 +1082,7 @@ class ZbxCommon(object):
                 fp.write("\n\n\n".join(infos))
         return tables_info
 
-    def dump_mysql_data(self, conn, dir='', limit=10, tables=[]):
+    def dump_mysql_data(self, conn, dir='', limit=10, tables=[],overwrite=False):
         # conn = self.get_mysql_connection(dsn)
         data_map = {}
         if len(tables) == 0:
@@ -1035,11 +1094,11 @@ class ZbxCommon(object):
                 sql = 'select * from `{table}`'.format(table=table, limit=limit)
             table_data = self.mysql_query(conn, sql)
             fpath = '{dir}/{table}.sql'.format(dir=dir, table=table)
-            sql_str = self.dict2sql(table, table_data)
+            sql_str = self.gen_insert_sql(table, table_data)
             if dir == '':
                 data_map[table] = sql_str
             if dir != '':
-                if not os.path.exists(fpath):
+                if not os.path.exists(fpath) or overwrite:
                     with open(fpath, 'w+') as fp:
                         fp.write(sql_str)
                         self.log.info('gen table {table}'.format(table=table))
@@ -1182,6 +1241,60 @@ class ZbxCommon(object):
         consumer = self.kafka.KafkaConsumer(info.dbname, **config)
         return consumer
 
+    def parse_fsm_flow(self, flow_define, action='trigger', _from='source', _to='dest'):
+        '''
+
+        :param flow_define:
+        :return:
+        '''
+        # # states = ['New' , 'Ready', 'Waiting', 'Running','Terminated']
+        # transitions = [
+        #     {'trigger': 'Admitted', 'source': 'New' , 'dest': 'Ready' },
+        #     {'trigger': 'Dispatch', 'source': 'Ready', 'dest': 'Running'},
+        #     {'trigger': 'Interrupt', 'source': 'Running', 'dest': 'Ready'},
+        #     {'trigger': 'InputOutputoreventwait', 'source': 'Running', 'dest': 'Waiting'},
+        #     {'trigger': 'InputOutputoreventcompletion', 'source': 'Waiting', 'dest': 'Ready'},
+        #     {'trigger': 'Exit', 'source': 'Running', 'dest': 'Terminated'}]
+        # flow=''''
+        # 1=New
+        # 2=Ready
+        # 3=Waiting
+        # 4=Running
+        # 5=Terminated
+        # Admitted:1->2
+        # Dispatch:2->4
+        # Interrupt:4->2
+        # InputOutputoreventwait:4->3
+        # InputOutputoreventcompletion:3->2
+        # Exit:4->5
+        # '''
+        # trim comment
+        flow_define = re.sub(r'^\s*//.*?\n|^\s*', '', flow_define, 0, re.MULTILINE)
+        lines = re.split(r'\n', flow_define)
+        step_map = {}
+        action_map = {}
+        actions = []
+        states = set()
+        for line in lines:  # parse step
+            kv = re.split(r'\s*=\s*', line, 1)
+            if len(kv) == 2:
+                step_map[kv[0]] = kv[1]
+        for line in lines:
+            m = re.match(r'(?P<action>[\s\S]+):(?P<from>[\s\S]+?)\s*->\s*(?P<to>[\s\S]+)', line)
+            if m is None:
+                continue
+            f = m.group('from')
+            t = m.group('to')
+            a = m.group('action')
+            if step_map.get(m.group('from')) is not None:
+                f = step_map.get(m.group('from'))
+            if step_map.get(m.group('to')) is not None:
+                t = step_map.get(m.group('to'))
+            states.add(f)
+            states.add(t)
+            actions.append({action: a, _from: f, _to: t})
+        return {'states': list(states), 'transitions': actions}
+
     @property
     def kafka(self):
         import kafka
