@@ -1,6 +1,5 @@
+# -*- coding:utf8 -*-
 from base import *
-
-
 # a=cli.render('''
 #    my name is {{name}}.
 #    {%- for i in [1,2,3,4,5,6,7] %}
@@ -18,7 +17,6 @@ from base import *
 # print(cli.mysql_query(conn,'select * from test'))
 
 
-
 # conn=cli.get_sqlite_connection(":memory:")
 #
 # cli.sqlite_query(conn,'''
@@ -58,7 +56,6 @@ from base import *
 # print(cli.get_dependencies())
 
 
-
 # tpl='''
 # <dl>
 #     {% for key, value in d.items() %}
@@ -97,7 +94,6 @@ from base import *
 #     print(i.attr('name'))
 
 
-
 # cli.example()
 
 
@@ -185,7 +181,7 @@ from base import *
 # print("response: %s" % response)
 
 
-cli.gen_requirements(".")
+# cli.gen_requirements()
 
 # app=cli.get_flask_app()
 # from flask import request
@@ -200,3 +196,65 @@ cli.gen_requirements(".")
 #
 # app.run('0.0.0.0',port=8000,debug=True)
 
+# states = ['New' , 'Ready', 'Waiting', 'Running','Terminated']
+# transitions = [
+#     {'trigger': 'Admitted', 'source': 'New', 'dest': 'Ready'},
+#     {'trigger': 'Dispatch', 'source': 'Ready', 'dest': 'Running'},
+#     {'trigger': 'Interrupt', 'source': 'Running', 'dest': 'Ready'},
+#     {'trigger': 'InputOutputoreventwait', 'source': 'Running', 'dest': 'Waiting'},
+#     {'trigger': 'InputOutputoreventcompletion', 'source': 'Waiting', 'dest': 'Ready'},
+#     {'trigger': 'Exit', 'source': 'Running', 'dest': 'Terminated'}]
+flow = '''
+  1=New
+2 = Ready
+  3=        Waiting
+4=Running
+5=Terminated
+Admitted:1->2
+Dispatch:2 -> 4
+Interrupt:4->2
+InputOutputoreventwait:4->3
+InputOutputoreventcompletion:3->2
+Exit:4->5
+'''
+#
+# flow = '''
+#
+# //格式如下：
+# 动作1:状态1 -> 状态2
+# 动作2:状态2 -> 状态3
+# 动作3:状态3 -> 状态4
+#
+# //举例
+# //通知member填写校招培训自评
+# NotifyMemberToFillCampusAssignment:init->WaitForMemberFillCampusAssignment
+# //member填写校招自评
+# MemberFillCampusAssignment:WaitingFillCampusAssignment->FinishFillCampusAssignment
+# //通知mentor提交校招生评价
+# NotifyMentorToFillCampusAssignment:FinishFillCampusAssignment->WaitForMentorFillCampusAssignment
+#
+# '''
+#
+# print(cli.parse_fsm_flow(flow))
+
+
+
+
+
+from transitions import Transition,Machine
+
+
+data=cli.parse_fsm_flow(flow)
+
+class Matter(object):
+    pass
+model = Matter()
+
+machine = Machine(model=model, **cli.parse_fsm_flow(flow), initial='New')
+
+# machine.set_state('Exit')
+for i in range(1,10):
+    print(model.state)
+    print(model.Admitted())
+    print(model.Dispatch())
+
